
#include <MouseSet.h>
#include <key.h>

static const char *StrMoTypeLabel = "Mouse type";
static const char *StrMoSpeedLabel = "Mouse speed";
static const char *StrMoSamplingLabel = "PS/2 sampling rate";
static const char *StrMoKbdLayout = "Keyboard layout";
	 
	 
CMouseSettings::CMouseSettings(CXOSLData &XoslDataToUse, CDialogs &DialogsToUse, CMouse &MouseToUse):
	XoslData(XoslDataToUse), Dialogs(DialogsToUse), Mouse(MouseToUse)
{
	Initialized = false;
}

CMouseSettings::~CMouseSettings()
{

}

void CMouseSettings::CreateControls()
{
	TypeGroup = new CBevel(BEVEL_FRAME,true,24,32,161,223,false);
	TypeLabel = new CLabel(StrMoTypeLabel,STYLE_REGULAR,false,17,32,25,false);
	MouseGroup = new CRadioGroup(56,50,120,144,false,this);
	ApplyBtn = new CButton("Apply",67,208,75,25,false,this);

	SpeedGroup = new CBevel(BEVEL_FRAME,true,200,32,241,41,false);
	SpeedLabel = new CLabel(StrMoSpeedLabel,STYLE_REGULAR,false,17,208,25,false);
	SpeedSlider = new CTrackBar(0,8,7,220,48,201,false,this);

	SamplingGroup = new CBevel(BEVEL_FRAME,true,200,96,241,41,false);
	SamplingLabel = new CLabel(StrMoSamplingLabel,STYLE_REGULAR,false,17,208,89,false);
	SamplingSlider = new CTrackBar(0,6,0,220,112,201,false,this);

	KeyboardGroup = new CBevel(BEVEL_FRAME,true,200,160,241,95,false);
	KeyboardLabel = new CLabel(StrMoKbdLayout,STYLE_REGULAR,false,17,208,153,false);
	LayoutList = new CListBox(1,false,224,177,193,49,false,this);
	ScrollBar = new CScrollBar(0,0,0,false,398,178,47,false,LayoutList);
	EnhKeyboard = new CCheckBox("Enhanced keyboard support",false,232,229,false,this);
}

void CMouseSettings::InitializeControls(CTabControl *TabControl)
{
	TabControl->AddControl(2,TypeGroup);
	TabControl->AddControl(2,TypeLabel);
	TabControl->AddControl(2,MouseGroup);
	TabControl->AddControl(2,SpeedGroup);
	TabControl->AddControl(2,SpeedLabel);
	TabControl->AddControl(2,SpeedSlider);
	TabControl->AddControl(2,SamplingGroup);
	TabControl->AddControl(2,SamplingLabel);
	TabControl->AddControl(2,SamplingSlider);
	TabControl->AddControl(2,ApplyBtn);
	TabControl->AddControl(2,KeyboardGroup);
	TabControl->AddControl(2,KeyboardLabel);
	TabControl->AddControl(2,LayoutList);
	TabControl->AddControl(2,ScrollBar);
	TabControl->AddControl(2,EnhKeyboard);


	MouseGroup->AddButton("Serial COM1");
	MouseGroup->AddButton("Serial COM2");
	MouseGroup->AddButton("Serial COM3");
	MouseGroup->AddButton("Serial COM4");
	MouseGroup->AddButton("PS/2");
	MouseGroup->AddButton("None");

	LayoutList->SetColumn(0,172,"");
	LayoutList->AddItem(LayoutList->AddRow(),0,"French");
	LayoutList->AddItem(LayoutList->AddRow(),0,"United States");
	LayoutList->SetScrollBar(ScrollBar);

}

void CMouseSettings::ConnectEventHandlers()
{
	SpeedSlider->OnChange((TTrackBarChange)SpeedChange);
	SamplingSlider->OnChange((TTrackBarChange)SamplingChange);
	ApplyBtn->OnClick((TWndOnClick)ApplyClick);
	LayoutList->OnSelect((TListBoxSelect)KbdLayoutSelect);
	EnhKeyboard->OnChange((TCheckBoxChange)KbdEnhChanges);
}

void CMouseSettings::InitializeData()
{
	TMouseData *MouseData;

	MouseData = XoslData.GetMouse();

	MouseGroup->SetChecked(MouseData->MouseType);
	SpeedSlider->SetValue(MouseData->MouseSpeed);
	SamplingSlider->SetValue(MouseData->PS2Sampling);
	SpeedChange(*this,MouseData->MouseSpeed);
	SamplingChange(*this,MouseData->PS2Sampling);

	InitKbdLayout(MouseData->KbdLayout);

	EnhKeyboard->SetChecked(MouseData->EnhancedSupport);

	Initialized = true;
}

void CMouseSettings::InitKbdLayout(int Layout)
{
	switch (Layout) {
		case CKeyboard::enumLayoutFrench:
			KbdLayoutSelect(*this,0);
			break;
		case CKeyboard::enumLayoutUS:
			KbdLayoutSelect(*this,1);
			break;
		default:
			break;
	}
}

void CMouseSettings::InstallControls(CForm *Form)
{
	PrefForm = Form;
	Form->AddControl(TypeGroup);
	Form->AddControl(TypeLabel);
	Form->AddControl(MouseGroup);
	Form->AddControl(ApplyBtn);
	Form->AddControl(SpeedGroup);
	Form->AddControl(SpeedLabel);
	Form->AddControl(SpeedSlider);
	Form->AddControl(SamplingGroup);
	Form->AddControl(SamplingLabel);
	Form->AddControl(SamplingSlider);
	Form->AddControl(KeyboardGroup);
	Form->AddControl(KeyboardLabel);
	Form->AddControl(LayoutList);
	Form->AddControl(ScrollBar);
	Form->AddControl(EnhKeyboard);

}

void CMouseSettings::RealignText()
{
	bool Visible;

	Visible = TypeLabel->IsVisible();
	
	TypeLabel->SetVisible(false);
	SpeedLabel->SetVisible(false);
	SamplingLabel->SetVisible(false);
	KeyboardLabel->SetVisible(false);
	
	TypeLabel->SetCaption(StrMoTypeLabel);
	SpeedLabel->SetCaption(StrMoSpeedLabel);
	SamplingLabel->SetCaption(StrMoSamplingLabel);
	KeyboardLabel->SetCaption(StrMoKbdLayout);
	
	TypeLabel->SetVisible(Visible);
	SpeedLabel->SetVisible(Visible);
	SamplingLabel->SetVisible(Visible);
}

// --------------------- event handlers --------------------- 

void CMouseSettings::SpeedChange(CMouseSettings &MouseSettings, int Value)
{
	if (MouseSettings.Initialized) {
		MouseSettings.Mouse.SetSpeed(Value);
		MouseSettings.XoslData.GetMouse()->MouseSpeed = Value;
	}
}

void CMouseSettings::SamplingChange(CMouseSettings &MouseSettings, int Value)
{
	if (MouseSettings.Initialized) {
		MouseSettings.Mouse.SetSampling(Value);
		MouseSettings.XoslData.GetMouse()->PS2Sampling = Value;
	}
}

void CMouseSettings::ApplyClick(CMouseSettings &MouseSettings)
{
	int LastPort;
	int MousePort;
	int Checked;
	CString Message;
	static const char *MsgBoxTitle = "Mouse";
	static const int PortList[] = {
		MOUSE_COM1,MOUSE_COM2,MOUSE_COM3,MOUSE_COM4,MOUSE_PS2
	};

	Graph->HideCursor();
	LastPort = MouseSettings.Mouse.GetMouse();
	Checked = MouseSettings.MouseGroup->GetChecked();
	if (Checked == 5) {
		MouseSettings.Mouse.SetMouse(-1);
		MouseSettings.XoslData.GetMouse()->MouseType = -1;
	}
	else {
		MousePort = PortList[Checked];
		Message = "Initializing ";
		Message += MouseSettings.Mouse.MousePortName(MousePort);
		Message += "...";
		MouseSettings.Dialogs.ShowMessageDialog(MouseSettings.PrefForm,MsgBoxTitle,Message);
		if (MouseSettings.Mouse.SetMouse(MousePort) == -1) {
			MouseSettings.Mouse.SetMouse(LastPort);
			Message += "failed!";
			MouseSettings.Dialogs.ShowMessageDialog(MouseSettings.PrefForm,MsgBoxTitle,Message);
			if (LastPort != -1)
				Graph->ShowCursor();
		}
		else {
			MouseSettings.XoslData.GetMouse()->MouseType = MousePort;
			Message += "done";
			MouseSettings.Dialogs.ShowMessageDialog(MouseSettings.PrefForm,MsgBoxTitle,Message);
			Graph->ShowCursor();
		}
		SpeedChange(MouseSettings,MouseSettings.SpeedSlider->GetValue());
		SamplingChange(MouseSettings,MouseSettings.SamplingSlider->GetValue());
	}
}

void CMouseSettings::KbdLayoutSelect(CMouseSettings &MouseSettings, int ItemIndex)
{
	CKeyboard::TKeyLayout Layout;

	switch (ItemIndex) {
		case 0: // French
			Layout = CKeyboard::enumLayoutFrench;
			break;
		case 1: // US
			Layout = CKeyboard::enumLayoutUS;
			break;
		default:
			return;
	}

	if (MouseSettings.Initialized) {
		CKeyboard::SelectLayout(Layout);
		MouseSettings.XoslData.GetMouse()->KbdLayout = Layout;
	}
	else {
		MouseSettings.LayoutList->SetItemIndex(ItemIndex);
	}
}

void CMouseSettings::KbdEnhChanges(CMouseSettings &MouseSettings, bool Checked)
{
	if (MouseSettings.Initialized) {
		MouseSettings.XoslData.GetMouse()->EnhancedSupport = Checked;
		CKeyboard::SetEnhancedSupport(Checked);
	}
}
